/**
 * Copyright (c) 2016-2019, Michael Yang 杨福海 (fuhai999@gmail.com).
 * <p>
 * Licensed under the GNU Lesser General Public License (LGPL) ,Version 3.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zx.dataservice.options;


import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * ws 一些常量 配置
 * ws_config 表
 * 2020-12-27
 *  by qlm
 */
public class WSOptions {

    private static Logger LOG = LoggerFactory.getLogger(WSOptions.class);

    private static OptionStore store = new OptionStore() {
        private final ConcurrentHashMap<String, String> cache = new ConcurrentHashMap<>();

        @Override
        public String get(String key) {
            return cache.get(key);
        }

        @Override
        public void put(String key, String value) {
            if (StringUtils.isBlank(value)) {
                remove(key);
            } else {
                cache.put(key, value);
            }
        }

        @Override
        public void remove(String key) {
            cache.remove(key);
        }

        public ConcurrentHashMap.KeySetView<String, String> keySet() {
            return cache.keySet();
        }
    };

    private static List<OptionChangeListener> LISTENERS = new ArrayList<>();

    public static void remove(String key) {
        store.remove(key);
    }
    public static  ConcurrentHashMap.KeySetView<String, String> keySet() {
        return store.keySet();
    }

    public static void set(String key, String value) {
        if (StringUtils.isBlank(key)) {
            return;
        }
        String oldValue = store.get(key);
        if (Objects.equals(value, oldValue)) {
            return;
        }
        store.put(key, value);
        for (OptionChangeListener listener : LISTENERS) {
            try {
                listener.onChanged(key, value, oldValue);
            } catch (Throwable ex) {
                LOG.error(ex.toString(), ex);
            }
        }
        doFinishedChanged(key, value, oldValue);
    }

    public static String get(String key) {
        return store.get(key);
    }

    public static String get(String key, String defaultvalue) {
        String v = get(key);
        return StringUtils.isBlank(v) ? defaultvalue : v;
    }

    public static boolean getAsBool(String key) {
        return Boolean.parseBoolean(store.get(key));
    }



    public static boolean isTrueOrNull(String key) {
        String data = get(key);
        return data == null || "true".equals(data);
    }

    public static int getAsInt(String key, int defaultValue) {
        String value = get(key);
        if (StringUtils.isBlank(value)) {
            return defaultValue;
        }
        try {
            return Integer.parseInt(value);
        } catch (Exception ex) {
            LOG.warn(ex.toString(), ex);
            return defaultValue;
        }
    }

    public static float getAsFloat(String key, float defaultValue) {
        String value = get(key);
        if (StringUtils.isBlank(value)) {
            return defaultValue;
        }
        try {
            return Float.parseFloat(value);
        } catch (Exception ex) {
            LOG.warn(ex.toString(), ex);
            return defaultValue;
        }
    }

    public static void addListener(OptionChangeListener listener) {
        LISTENERS.add(listener);
    }

    public static void removeListener(OptionChangeListener listener) {
        LISTENERS.remove(listener);
    }


    public static interface OptionChangeListener {
        public void onChanged(String key, String newValue, String oldValue);
    }


    private static void doFinishedChanged(String key, String value, String oldValue) {


    }

    private static boolean fakeStaticEnable = false;
    private static String fakeStaticSuffix = "";

    public static String getAppUrlSuffix() {
        return fakeStaticEnable
                ? (StringUtils.isBlank(fakeStaticSuffix) ? "" : fakeStaticSuffix)
                : "";
    }

    public static OptionStore getStore() {
        return store;
    }


    public static interface OptionStore {

        public String get(String key);

        public void put(String key, String value);

        public void remove(String key);

        public ConcurrentHashMap.KeySetView<String, String> keySet();

    }
}
